from django.utils import timezone
from django.db import models
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from mimetypes import guess_type
from crea_app.storages import UthoStorage
from .utils import headshot_upload_to, post_image_upload_to, post_thumbnail_upload_to, post_video_upload_to
from django.db.models.signals import post_save
from django.dispatch import receiver

User = get_user_model()

def validate_media_type(file):
    mime_type, _ = guess_type(file.name)
    if mime_type:
        if not (mime_type.startswith('image') or mime_type.startswith('video')):
            raise ValidationError("Only image or video files are allowed.")
    else:
        raise ValidationError("Cannot determine the file type. Only images and videos are allowed.")
    
class SoftDeleteManager(models.Manager):
    """Custom manager to exclude soft-deleted objects by default."""
    def get_queryset(self):
        return super().get_queryset().filter(is_deleted=False)

class SoftDeleteModel(models.Model):
    """Abstract base class for soft delete functionality."""
    is_deleted = models.BooleanField(default=False)
    deleted_at = models.DateTimeField(blank=True, null=True)

    objects = SoftDeleteManager()  # Default manager excludes deleted objects
    all_objects = models.Manager()  # Include deleted objects

    def delete(self, *args, **kwargs):
        """Soft delete the object."""
        self.is_deleted = True
        self.deleted_at = timezone.now()
        self.save()

    def restore(self, *args, **kwargs):
        """Restore a soft-deleted object."""
        self.is_deleted = False
        self.deleted_at = None
        self.save()

    class Meta:
        abstract = True

class Post(SoftDeleteModel): 
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='posts')
    caption = models.TextField(blank=True, null=True)
    location = models.CharField(max_length=255, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)
    thumbnail = models.ImageField(upload_to='thumbnails/', null=True, blank=True)

    def __str__(self):
        return f"{self.user.username} - {self.caption[:20]}"

class PostMedia(SoftDeleteModel):
    MEDIA_TYPE_CHOICES = [
        ('image', 'Image'),
        ('video', 'Video'),
    ]
    post = models.ForeignKey(Post, on_delete=models.CASCADE, related_name='media_files')
    media = models.FileField(
        storage=UthoStorage(),
        upload_to=post_image_upload_to,
        validators=[validate_media_type],
    )
    media_type = models.CharField(max_length=10, choices=MEDIA_TYPE_CHOICES)

    def save(self, *args, **kwargs):
        # Determine the file type
        mime_type, _ = guess_type(self.media.name)
        if mime_type:
            if mime_type.startswith('image'):
                self.media_type = 'image'
                self.media.field.upload_to = post_image_upload_to
            elif mime_type.startswith('video'):
                self.media_type = 'video'
                self.media.field.upload_to = post_video_upload_to
            else:
                raise ValidationError("Only image or video files are allowed.")
        else:
            raise ValidationError("Cannot determine the file type. Only images and videos are allowed.")
        super().save(*args, **kwargs)



class Like(SoftDeleteModel):
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='likes')
    post = models.ForeignKey(Post, on_delete=models.CASCADE, related_name='likes')
    created_at = models.DateTimeField(default=timezone.now)

    class Meta:
        unique_together = ('user', 'post')


class Comment(SoftDeleteModel):
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='comments')
    post = models.ForeignKey(Post, on_delete=models.CASCADE, related_name='comments')
    parent = models.ForeignKey('self', on_delete=models.CASCADE, null=True, blank=True, related_name='replies')
    content = models.TextField()
    created_at = models.DateTimeField(default=timezone.now)
    likes = models.ManyToManyField(User, related_name='comment_likes', blank=True)

    def __str__(self):
        if self.parent:
            return f"Reply by {self.user.username}: {self.content[:20]}"
        return f"Comment by {self.user.username}: {self.content[:20]}"


class Headshot(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='headshots')
    banner = models.ImageField(
        storage=UthoStorage(),
        upload_to=headshot_upload_to,
        validators=[validate_media_type]
    )
    film_name = models.CharField(max_length=255)
    role_played = models.CharField(max_length=255)
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"{self.film_name} - {self.role_played} ({self.user.username})"

class ReportReason(models.Model):
    reason = models.CharField(max_length=255, unique=True)

    def __str__(self):
        return self.reason


class PostReport(models.Model):
    post = models.ForeignKey('Post', on_delete=models.CASCADE, related_name='reports')
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='reports')
    reason = models.ForeignKey(ReportReason, on_delete=models.CASCADE, related_name='reports')
    reported_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        unique_together = ('post', 'user')  # Prevent duplicate reports by the same user.

    def __str__(self):
        return f"Report by {self.user.username} on Post ID {self.post.id} for {self.reason}"


@receiver(post_save, sender=PostReport)
def handle_post_report(sender, instance, created, **kwargs):
    if created:
        report_count = instance.post.reports.count()
        if report_count >= 50 and not instance.post.is_deleted:
            instance.post.delete()